@@ -25,6 +25,10 @@
  *   Olaf Mersmann <olafm@statistik.tu-dortmund.de>
  */
 
+#include <float.h>
+
+#define STRICT_R_HEADERS 1
+
 #include <R.h>
 #include <Rmath.h>
 #include <Rinternals.h>
